/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 * 
 * Copyright (C) 2012  Eva Mujdricza-Maydt
 * Copyright (C) 2012  Huiqin Körkel-Qu
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 * Developers:
 * - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de)
 * - Huiqin Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
 
package corpora.opensubtitles;

import java.util.ArrayList;
import objects.Sentence;
import objects.Word;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/** Handler for extracting the text from the original, monolingual xml-files into a
 * <code>Sentence</code> list.
 *
 * @author Eva Mujdricza-Maydt
 * @lastChange 20120626
 */
public class XMLHandler_opensubtitles_monolingualXML extends DefaultHandler {

	private ArrayList<Sentence> text = null;
	private Sentence currSent = null;
	private ArrayList<Word> currWords = null;
	private Word currWord = null;

	private boolean withinCurrentWord = false;

	//because of "character chunking"
	//see also: http://www.tutego.de/blog/javainsel/2007/01/character-chunking-bei-sax.html
	private static StringBuffer wordBuffer = null;

	@Override
	public void startDocument() throws SAXException {
		text = new ArrayList<Sentence>();
	}

	@Override
	public void endDocument() throws SAXException {
		ExtractOpensubltitles.monolingualText = text;
	}

	@Override
	public void startElement(String namespaceURI, String localName,
		String qName, Attributes atts) throws SAXException {

		if (qName.equals(Constants_OpenSubtitles.ELEMENT_s)) {
			currSent = new Sentence();
			currSent.setSentenceNr(Integer.parseInt(atts.getValue(Constants_OpenSubtitles.ATTRIBUTE_id)));
			currWords = new ArrayList<Word>();
		}
		else if (qName.equals(Constants_OpenSubtitles.ELEMENT_w)) {
			currWord = new Word();
			String[] raw_sentNrWordNr = atts.getValue(Constants_OpenSubtitles.ATTRIBUTE_id).split("\\.");
			int sentNr = Integer.parseInt(raw_sentNrWordNr[0]);
			int wordNr = Integer.parseInt(raw_sentNrWordNr[1]);
			currWord.setSentenceNr(sentNr);
			currWord.setWordInSentNr(wordNr);

			wordBuffer = new StringBuffer();
			withinCurrentWord = true;
		}	
	}

	@Override
	public void endElement(String namespaceURI, String localName,
		String qName) throws SAXException {

		if (qName.equals(Constants_OpenSubtitles.ELEMENT_s)) {
			currSent.setWordList(currWords);
			text.add(currSent);

			currWords = null;
			currSent = null;
		}
		else if (qName.equals(Constants_OpenSubtitles.ELEMENT_w)) {
			currWord.setWord(wordBuffer.toString().trim());
			currWords.add(currWord);

			wordBuffer = null;
			currWord = null;
			withinCurrentWord = false;
		}
	}

	@Override
	public void characters(char[] ch, int startPos, int length) {

		if (withinCurrentWord == true) {
			wordBuffer.append(new String(ch, startPos, length));
		}
	}

}
