/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package diceMatrix;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

/**
 * read the dice matrix file which contains how many times a word pair appears,
 * when both source and target word are often words(both appear many times in
 * training data, the number of often words is decide by myConst.maxSM, and
 * myConst.maxTM, default value are both 4000)
 *
 * @author Huiqin Körkel-Qu
 *
 */
public class getDiceMatrix {

	/**
	 *
	 *
	 * @param fileName
	 * @return diceMatrix, which is int[myConst.maxSM][myConst.maxTM],
	 * diceMatrix[i][j] is how many times the word pair (i,j) appears.
	 *
	 * @throws NumberFormatException
	 * @throws IOException
	 */
	public static int[][] get(String fileName) throws NumberFormatException, IOException {

		File diceFile = new File(fileName);

		int[][] diceMatrix = new int[myConst.maxSM][myConst.maxTM];
		System.out.println("dice size is" + myConst.maxSM + " and " + myConst.maxTM);

		BufferedReader BR = new BufferedReader(new FileReader(diceFile));

		int lineNum = 0;
		String currentLine;
		while (null != (currentLine = BR.readLine()) && lineNum < myConst.maxSM) {

			String[] times = currentLine.split("\\s+");
			for (int i = 0; i < Math.min(times.length, myConst.maxTM); i++) {
//				System.out.println("sourceNum and targetNum is "+ lineNum +" and  "+i);
				diceMatrix[lineNum][i] = Integer.parseInt(times[i]);
			}
			lineNum += 1;
		}

		BR.close();
		return diceMatrix;
	}
}
