/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package diceMatrix.train;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import diceMatrix.getDiceMatrix;
import diceMatrix.getMultiWordsHash;
import diceMatrix.getSingleHash;
import diceMatrix.myConst;
import diceMatrix.sentencePairDice;
import diceMatrix.wordPairDice;

/**
 * get the dice for all the training file pairs
 *
 * @author Huiqin Körkel-Qu
 */
public class allFileDice {

	public static void main(String[] args) throws IOException, InterruptedException {

		String filePath = args[0];
//		System.out.println(filePath);
		String alignPath = filePath + "align/";
		String sourceNumPath = filePath + "num/de/";
		String targetNumPath = filePath + "num/en/";
		String dicePath = filePath + "dice/";

		File diceDir = new File(dicePath);
		if (!diceDir.isDirectory()) {
			boolean sucess = diceDir.mkdirs();

			if (!sucess) {
				System.out.println(diceDir.getAbsolutePath());
				System.out.println("uncessful in creating dice dir!");
				System.exit(1);
			}
		}

		HashMap<Integer, Integer> sourceMultiHash = getMultiWordsHash.getMultiNumTimesHash(filePath + "source.countmulti");
		HashMap<Integer, Integer> targetMultiHash = getMultiWordsHash.getMultiNumTimesHash(filePath + "target.countmulti");

		int[] sourceSingleWords = getSingleHash.getNum(filePath + "source.countsingle");
		int[] targetSingleWords = getSingleHash.getNum(filePath + "target.countsingle");
		int[][] diceMatrix = getDiceMatrix.get(filePath + "dice.matrix");
		ArrayList<HashMap<Integer, Integer>> diceArray = getDiceArray.get(filePath + "source.array");

		myConst.fastSourceWordsNum = sourceMultiHash.size() + sourceSingleWords.length;
		myConst.fastTargetWordsNum = targetMultiHash.size() + targetSingleWords.length;

//		System.out.println("there are "+myConst.fastSourceWordsNum+" source tokens");
//		System.out.println("there are "+myConst.fastTargetWordsNum+" target tokens");

		wordPairDice wordDice = new wordPairDice(sourceMultiHash, targetMultiHash, diceMatrix, diceArray, null);
		sentencePairDice sentDice = new sentencePairDice(wordDice);
		fileDiceTrain fileDice = new fileDiceTrain(sentDice);

		File sourceDir = new File(sourceNumPath);
		File[] sourceFiles = sourceDir.listFiles();

		File[] alignFileList = new File(alignPath).listFiles();
//	    System.out.println(alignPath);
//	    System.out.println();
		if (null == alignFileList || alignFileList.length == 0) {
			System.out.println("the alignmentPath is empty!");
			System.exit(1);
		}
//	    for(File file:alignFileList)
//	    {
//	    	System.out.println(file.getAbsolutePath());
//	    }

//	    long diceStart=System.currentTimeMillis();
		int diceFileNum = 0;
		for (File sourceFile : sourceFiles) {
//	    	System.out.println("calculate dice for file "+sourceFile.getAbsolutePath());
//	    	long diceStart=System.currentTimeMillis();
			File targetFile = new File(targetNumPath + sourceFile.getName());

			File alignFile = new File(alignPath + sourceFile.getName().split("\\.")[0] + myConst.extensionForAlign);

//	    	System.out.println("alignFile is "+alignFile.getAbsolutePath());
			File diceFile = new File(dicePath + sourceFile.getName().split("\\.")[0] + ".dice");
			if (!targetFile.isFile() || !alignFile.isFile()) {

//	    		System.out.println("target file or alignFile is not there!");
//	    		System.out.println(targetFile.getAbsolutePath());
//	    		System.out.println(alignFile.getAbsolutePath());
				continue;
			}

			/*
			 * Dice for one single traing file pair.
			 */
			fileDice.get(sourceFile, targetFile, alignFile, diceFile);
			diceFileNum += 1;

//	    	if(i>1)
//	    	{
//	    		break;
//	    	}

//	         System.out.println("for file "+sourceFile.getName()+ "\nthe dice value calculation costed  "+(diceEnd-diceStart)+"mini seconds");

		}

//	    System.out.println("there are "+diceFileNum+" dice files");
		if (diceFileNum == 0) {
//	    	System.out.println("the alignappenddice is "+alignAppendix);
			System.out.println("Error! No dice files is created!");
			System.exit(1);
		}

//	    long diceEnd=System.currentTimeMillis();

//        System.out.println("it costs" +(diceEnd-diceStart)/1000+" seconds get the dice feature for training data");
		System.out.println("get all dice values for training data");

		String moveFeatureCommand = "rm -r " + args[0] + "num/";
		Process moveFeatureDir = Runtime.getRuntime().exec(moveFeatureCommand);
		moveFeatureDir.waitFor();
	}
}
