/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package diceMatrix.train;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;

import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import diceMatrix.getSent;
import diceMatrix.myConst;

/**
 * calculate how many times a pair of words appear. the words from
 * source(target) are seperate into 3 groups often words(the first
 * myConst.maxSM(3000) words that appear most often) commen words(the first
 * myConst.fastSourceWordsNum(10000) words that appear most often) rare words,
 * all the words.
 *
 * so the source target wordpairs are separated into 3 area 1 both are often
 * words, a matrix of size myConst.maxSM*myConst.maxTM will be created. the
 * element diceMatrix[i][j] saves how often the wordpair i-j appears 2 one is
 * commen word(source or target). e.g if source word i is a common word. the
 * sourceToTargetList.get(i) contains target word j. the
 * targetToSourceList.get(j) contains source word i.
 *
 * @author Huiqin Körkel-Qu
 */
public class diceMatrixList {

	//source and target are both often words
	int[][] diceMatrix;
	//source is commen word
	ArrayList<HashMap<Integer, Integer>> sourceToTargetList;
	//source or target is rare word.
	String rarePath;
	HashMap<Integer, Integer> sourceMultiNumHash;
	HashMap<Integer, Integer> targetMultiNumHash;
	int sourceWordsNum;
	int targetWordsNum;
	String[] singleSource;
	String[] singleTarget;

	public diceMatrixList(HashMap<Integer, Integer> sourceMultiNumHash, HashMap<Integer, Integer> targetMultiNumHash, String[] sourceSingle,
		String[] targetSingle) throws IOException {

		this.sourceWordsNum = sourceMultiNumHash.size() + sourceSingle.length;
		this.targetWordsNum = targetMultiNumHash.size() + targetSingle.length;
		this.sourceMultiNumHash = sourceMultiNumHash;
		this.targetMultiNumHash = targetMultiNumHash;

		sourceToTargetList = new ArrayList<HashMap<Integer, Integer>>();
		myConst.fastSourceWordsNum = sourceWordsNum;
		myConst.fastTargetWordsNum = targetWordsNum;
		diceMatrix = new int[Math.min(sourceWordsNum, myConst.maxSM)][Math.min(targetWordsNum, myConst.maxTM)];
		if (sourceWordsNum > myConst.maxSM || targetWordsNum > myConst.maxSM) {
			for (int i = 0; i < (Math.min(myConst.fastSourceWordsNum, sourceWordsNum)); i++) {
				HashMap<Integer, Integer> orginal = new HashMap<Integer, Integer>(2);
				sourceToTargetList.add(orginal);
			}
		}
	}

	public boolean calculateAppearance(ArrayList<int[]> sourceSentList, ArrayList<int[]> targetSentList,
		ArrayList<int[]> sourceAlign, ArrayList<int[]> targetAlign) throws IOException {
		//System.out.println("run calculatAppearance");

		if (sourceAlign.size() != targetAlign.size()) {
			System.out.println("source alignment and target alignment are not equal!");
			return false;
		}

		for (int i = 0; i < sourceAlign.size(); i++) {
			/*
			 * System.out.println("the "+i+"th alignment is:" ); for(int
			 * j=0;j<sourceAlign.get(i).length;j++) {
			 * System.out.print(sourceAlign.get(i)[j]+" "); }
			 * System.out.println(";"); for(int
			 * j=0;j<targetAlign.get(i).length;j++) {
			 * System.out.print(targetAlign.get(i)[j]+" "); }
			System.out.println();
			 */

			int[] sourceSent = getSent.get(sourceSentList, sourceAlign.get(i));
			int[] targetSent = getSent.get(targetSentList, targetAlign.get(i));
			this.calculateOneAlign(sourceSent, targetSent);
		}
		//System.out.println(count+" times oneAlign");
		return true;
	}

	
	// the count the appearance of words pair for one sentence alignment.
	@SuppressWarnings("unchecked")
	public void calculateOneAlign(int[] sourceSent, int[] targetSent) throws IOException {
		//because of the stop words, it can happen that a sentence contains only stop words, and after remove stop words
		//the sentence is empty. 
		if (null == sourceSent || null == targetSent) {
			return;
		}
//		System.out.print("source is: ");
//		for(int i=0;i<sourceSent.length;i++)
//			System.out.print(sourceSent[i]+" ");
//		System.out.println();
//		
//		System.out.print("target is: ");
//		for(int i=0;i<targetSent.length;i++)
//			System.out.print(targetSent[i]+" ");
//		System.out.println();
//		

		// save every token and the times it appears in source and target.
		HashMap<Integer, Integer> sourceTokenTimes = new HashMap<Integer, Integer>();
		HashMap<Integer, Integer> targetTokenTimes = new HashMap<Integer, Integer>();

		for (int i = 0; i < sourceSent.length; i++) {

			if (null == (sourceTokenTimes.get(sourceSent[i]))) {
				sourceTokenTimes.put(sourceSent[i], 1);

			} else {
				int times = sourceTokenTimes.get(sourceSent[i]) + 1;
				sourceTokenTimes.put(sourceSent[i], times);

			}
		}

		for (int i = 0; i < targetSent.length; i++) {

			if (null == (targetTokenTimes.get(targetSent[i]))) {
				targetTokenTimes.put(targetSent[i], 1);

			} else {
				int times = targetTokenTimes.get(targetSent[i]) + 1;
				targetTokenTimes.put(targetSent[i], times);

			}
		}

		//oftenTargets saves all the often target words appeared in target sent
		ArrayList<Integer> oftenTargets = new ArrayList<Integer>();
		//oftenTargets saves all the commen target words appeared in target sent
		ArrayList<Integer> commenTargets = new ArrayList<Integer>();
		//oftenTargets saves all the rare target words appeared in target sent
		ArrayList<Integer> rareTargets = new ArrayList<Integer>();

		//separate the target words into often, commen and rare.
		Iterator targetIter = targetTokenTimes.entrySet().iterator();
		while (targetIter.hasNext()) {
			Map.Entry target = (Map.Entry) targetIter.next();
			int targetWord = (Integer) target.getKey();
			@SuppressWarnings("unused")
			int targetTimes = (Integer) target.getValue();
			//target is a word which never appears
			if (targetWord == -1) {
				continue;
			}
			if (0 <= targetWord && targetWord < myConst.maxTM) {
				oftenTargets.add(targetWord);

			} else if (targetWord < myConst.fastTargetWordsNum) {
				commenTargets.add(targetWord);
			} else {
				rareTargets.add(targetWord);
			}
		}

		//oftenSources saves all the often Source words appeared in Target sent
		ArrayList<Integer> oftenSources = new ArrayList<Integer>();
		//oftenTargets saves all the commen Target words appeared in Target sent
		ArrayList<Integer> commenSources = new ArrayList<Integer>();
		//oftenTargets saves all the rare Target words appeared in Target sent
		ArrayList<Integer> rareSources = new ArrayList<Integer>();

		Iterator sourceIter = sourceTokenTimes.entrySet().iterator();

		//separate the source words into often, commen and rare.
		while (sourceIter.hasNext()) {
			Map.Entry source = (Map.Entry) sourceIter.next();
			int sourceWord = (Integer) source.getKey();
			@SuppressWarnings("unused")
			int sourceTimes = (Integer) source.getValue();
			//source is a word which never appears
			if (sourceWord == -1) {
				continue;
			}
			if (0 <= sourceWord && sourceWord < myConst.maxSM) {
				oftenSources.add(sourceWord);

			} else if (sourceWord < myConst.fastSourceWordsNum) {
				commenSources.add(sourceWord);
			} else {
				rareSources.add(sourceWord);
			}
		}

		this.writeSourceOften(oftenSources, oftenTargets, commenTargets, sourceTokenTimes, targetTokenTimes);
		this.writeSourceCommen(commenSources, oftenTargets, commenTargets, sourceTokenTimes, targetTokenTimes);
		this.writeSourceRare(rareSources, sourceTokenTimes, targetTokenTimes);
		this.writeTargetRare(commenSources, rareSources, rareTargets, sourceTokenTimes, targetTokenTimes);
	}

	@SuppressWarnings("unchecked")
	public void writeSourceRare(ArrayList<Integer> rareSources,
		HashMap<Integer, Integer> sourceTokenTimes, HashMap<Integer, Integer> targetTokenTimes) throws IOException {
		// all rare source word, write all targets into file!

		//find out the minimal appear times of target words
		int minTargetTimes = 0;
		Iterator targetTokenTimesIt = targetTokenTimes.entrySet().iterator();
		while (targetTokenTimesIt.hasNext()) {
			Map.Entry target = (Map.Entry) targetTokenTimesIt.next();
			minTargetTimes = Math.max((Integer) target.getValue(), minTargetTimes);
		}

		for (Integer sourceWord : rareSources) {
			//if all the target words appear times are more than 10 times the appear times for this rare source word,
			//the rare word should not be from a word pair which target word is from the target words list in this sentence.
			if (sourceTokenTimes.get(sourceWord) * myConst.maxDiff < minTargetTimes) {
				continue;
			}
			int sourceRareMod = sourceWord % myConst.rareFileNum;
			int sourceRareOrder = sourceWord / myConst.rareFileNum;


			File rareSource = new File(rarePath + "source." + sourceRareMod);
			BufferedReader sourceBr = new BufferedReader(new FileReader(rareSource));

			//copy the corresponding raresourceFile, and find the line which saves the words sourceSent[i];
			ArrayList<String> keepLine = new ArrayList<String>();
			String currentLine;
			while (null != (currentLine = sourceBr.readLine())) {
				keepLine.add(currentLine);
			}

			sourceBr.close();

			// take the line out which saves the information of sourceWord
			String[] currentAppearance = keepLine.get(sourceRareOrder).split("\\s+");
			HashMap<Integer, Integer> targetHash = new HashMap<Integer, Integer>();

			if (Integer.parseInt(currentAppearance[0]) != sourceWord) {
				System.out.println("the supposed wordNum is" + sourceWord
					+ "\n but the real word is " + currentAppearance[0]);
				continue;
			}
			// targetHash saves the targetword and how many times the word pair [sourceWord,targetWord] appears for sourceWord.
			for (int j = 1; j < currentAppearance.length / 2 + 1; j++) {
				targetHash.put(Integer.parseInt(currentAppearance[j * 2 - 1]), Integer.parseInt(currentAppearance[j * 2]));
			}

			Iterator targetTokenIt = targetTokenTimes.entrySet().iterator();
			while (targetTokenIt.hasNext()) {

				Entry target = (Map.Entry) targetTokenIt.next();
				Integer targetWord = (Integer) target.getKey();
				@SuppressWarnings("unused")
				int targetTimes = (Integer) target.getValue();

				int sourceTimes;
				if (null != sourceMultiNumHash.get(sourceWord)) {
					sourceTimes = sourceMultiNumHash.get(sourceWord);
				} else {
					sourceTimes = 1;
				}

				//if the target words appear much more often than the rare source word, it should not be a wordpair.
				if (sourceTimes * myConst.maxDiff < targetMultiNumHash.get(Math.min(targetWord, targetMultiNumHash.size() - 1))) {
					continue;
				}

				if (null == targetHash.get(targetWord)) {
					int times = Math.min(sourceTokenTimes.get(sourceWord), targetTokenTimes.get(targetWord));
					targetHash.put(targetWord, times);
				} else {
					int oldtimes = targetHash.get(targetWord);
					int times = Math.min(sourceTokenTimes.get(sourceWord), targetTokenTimes.get(targetWord));
					targetHash.put(targetWord, times + oldtimes);
				}
			}

			//get the new line for the rare source word.
			StringBuilder newAppearance = new StringBuilder();
			newAppearance.append(sourceWord + "");
			Iterator targetHashIter = targetHash.entrySet().iterator();
			while (targetHashIter.hasNext()) {
				Map.Entry targetInHash = (Map.Entry) targetHashIter.next();
				Integer targetHashWord = (Integer) targetInHash.getKey();
				int targetHashTimes = (Integer) targetInHash.getValue();
				newAppearance.append(" " + targetHashWord);
				newAppearance.append(" " + targetHashTimes);
			}
			keepLine.set(sourceRareOrder, newAppearance.toString());
			//write back
			BufferedWriter sourceBw = new BufferedWriter(new FileWriter(rareSource));
			for (int j = 0; j < keepLine.size(); j++) {
				sourceBw.write(keepLine.get(j) + "\n");
				//System.out.println("now write line:"+keepLine.get(j));
			}
			sourceBw.flush();
			sourceBw.close();
		}
	}

	@SuppressWarnings("unchecked")
	public void writeTargetRare(ArrayList<Integer> commenSources, ArrayList<Integer> rareSources, ArrayList<Integer> rareTargets,
		HashMap<Integer, Integer> sourceTokenTimes, HashMap<Integer, Integer> targetTokenTimes) throws IOException {

		int minSourceCommenTimes = 0;
		for (Integer sourceWord : commenSources) {
			int times;
			if (null != sourceMultiNumHash.get(sourceWord)) {
				times = sourceMultiNumHash.get(sourceWord);
			} else {
				times = 1;
			}
			minSourceCommenTimes = Math.min(minSourceCommenTimes, times);
		}
		//writeTaregetRare write only the case target is rare, source is not rare.

		for (Integer targetWord : rareTargets) {
			//if all the target words appear times are more than maxDiff times the appear times for this rare source word,
			//the rare word should not be from a word pair which target word is from the target words list in this sentence.
			int targetTimes;
			if (null != targetMultiNumHash.get(targetWord)) {
				targetTimes = targetMultiNumHash.get(targetWord);
			} else {
				targetTimes = 1;
			}
			if (targetTimes * myConst.maxDiff < minSourceCommenTimes) {
				continue;
			}
			Integer targetRareMod = targetWord % myConst.rareFileNum;
			int targetRareOrder = targetWord / myConst.rareFileNum;

			File rareTarget = new File(rarePath + "target." + targetRareMod);
			BufferedReader targetBr = new BufferedReader(new FileReader(rareTarget));

			//copy the corresponding raresourceFile, and find the line which saves the words sourceSent[i];
			ArrayList<String> keepLine = new ArrayList<String>();
			String currentLine;
			while (null != (currentLine = targetBr.readLine())) {
				keepLine.add(currentLine);
			}

			targetBr.close();

			// take the line out which saves the information of sourceWord
			String[] currentAppearance = keepLine.get(targetRareOrder).split("\\s+");
			HashMap<Integer, Integer> sourceHash = new HashMap<Integer, Integer>();
//	    			
//					 if(Integer.parseInt(currentAppearance[0])!=targetWord)
//						 {
//							 System.out.println("the supposed wordNum is" + targetWord+
//									 "\n but the real word is "+currentAppearance[0]);
//							 continue;
//						 }

			// targetHash saves the targetword and how many times the word pair [sourceWord,targetWord] appears for sourceWord.
			for (int j = 1; j < currentAppearance.length / 2 + 1; j++) {
				sourceHash.put(Integer.parseInt(currentAppearance[j * 2 - 1]), Integer.parseInt(currentAppearance[j * 2]));
			}

			Iterator sourceTokenTimesIt = sourceTokenTimes.entrySet().iterator();
			while (sourceTokenTimesIt.hasNext()) {
				Map.Entry source = (Map.Entry) sourceTokenTimesIt.next();
				int sourceWord = (Integer) source.getKey();
				//if source word is a rare word, the pair has been writen to rare source part.
				if (rareSources.contains(sourceWord) || sourceWord == -1) {
					continue;
				}
				@SuppressWarnings("unused")
				int sourceTimes = (Integer) source.getValue();
				//if the source word appears much more often than the rare target word, it should not be a wordpair.
				int targetLocalTimes;
				if (null != targetMultiNumHash.get(targetWord)) {
					targetLocalTimes = targetMultiNumHash.get(targetWord);
				} else {
					targetLocalTimes = 1;
				}
				int sourceMultiSize = sourceMultiNumHash.size();
				if (sourceMultiSize == 0) {
					System.out.println("the sourceMulti is empty!");
				}
				try {
					if (targetLocalTimes * myConst.maxDiff < sourceMultiNumHash.get(Math.min(sourceWord, sourceMultiSize - 1))) {
						continue;
					}
				} catch (Exception o) {
					System.out.println("source word is " + sourceWord);
					o.printStackTrace();
					System.exit(1);
				}

				if (null == sourceHash.get(sourceWord)) {
					int times = Math.min(targetTokenTimes.get(targetWord), sourceTokenTimes.get(sourceWord));
					sourceHash.put(sourceWord, times);
				} else {
					int oldtimes = sourceHash.get(sourceWord);
					int times = Math.min(targetTokenTimes.get(targetWord), sourceTokenTimes.get(sourceWord));
					sourceHash.put(sourceWord, times + oldtimes);
				}
			}

			//get the new line for the rare source word.
			StringBuilder newAppearance = new StringBuilder();
			newAppearance.append(targetWord + "");
			Iterator sourceHashIter = sourceHash.entrySet().iterator();
			while (sourceHashIter.hasNext()) {
				Map.Entry sourceInHash = (Map.Entry) sourceHashIter.next();
				int sourceHashWord = (Integer) sourceInHash.getKey();
				int sourceHashTimes = (Integer) sourceInHash.getValue();
				newAppearance.append(" " + sourceHashWord);
				newAppearance.append(" " + sourceHashTimes);
			}
			keepLine.set(targetRareOrder, newAppearance.toString());
			//write back
			BufferedWriter targetBw = new BufferedWriter(new FileWriter(rareTarget));
			for (int j = 0; j < keepLine.size(); j++) {
				targetBw.write(keepLine.get(j) + "\n");
				//System.out.println("now write line:"+keepLine.get(j));
			}
			targetBw.flush();
			targetBw.close();
		}
	}

	public void writeSourceOften(ArrayList<Integer> oftenSources, ArrayList<Integer> oftenTargets,
		ArrayList<Integer> commenTargets, HashMap<Integer, Integer> sourceTokenTimes,
		HashMap<Integer, Integer> targetTokenTimes) {
		//all the often source words
		for (Integer sourceWord : oftenSources) {
			//all the often target words
			for (int i = 0; i < oftenTargets.size(); i++) {
				int targetWord = oftenTargets.get(i);
				diceMatrix[sourceWord][targetWord] +=
					Math.min(sourceTokenTimes.get(sourceWord), targetTokenTimes.get(targetWord));
			}

			//the source word appears very often, it is nearly a stop words. such as ',',  'lieben'. so 
			//no commen target list for such source words
			@SuppressWarnings("unused")
			int sourceTimes;
			if (null != sourceMultiNumHash.get(sourceWord)) {
				sourceTimes = sourceMultiNumHash.get(sourceWord);
			} else {
				sourceTimes = 1;
			}
			if (sourceMultiNumHash.get(sourceWord) > 5
				* targetMultiNumHash.get(Math.min(targetMultiNumHash.size() - 1, myConst.maxTM - 1))) {
				continue;
			}
			// all the commen target words
			HashMap<Integer, Integer> commenTargetHash = sourceToTargetList.get(sourceWord);

			for (int i = 0; i < commenTargets.size(); i++) {
				int targetWord = commenTargets.get(i);
				//source and target word appear times are very different. omit it.
				if (null != targetMultiNumHash.get(targetWord) && null != sourceMultiNumHash.get(sourceWord)) {
					if (myConst.maxOftenCommenDiff * targetMultiNumHash.get(targetWord) < sourceMultiNumHash.get(sourceWord)) {
						continue;
					}
				}
				if (null == targetMultiNumHash.get(targetWord) && null != sourceMultiNumHash.get(sourceWord)) {
					if (myConst.maxOftenCommenDiff < sourceMultiNumHash.get(sourceWord)) {
						continue;
					}
				}

				if (null == commenTargetHash.get(targetWord)) {
					int times = Math.min(sourceTokenTimes.get(sourceWord), targetTokenTimes.get(targetWord));
					commenTargetHash.put(targetWord, times);
				} else {
					int times = Math.min(sourceTokenTimes.get(sourceWord), targetTokenTimes.get(targetWord));
					int oldTimes = commenTargetHash.get(targetWord);
					commenTargetHash.put(targetWord, oldTimes + times);
				}
			}
			sourceToTargetList.set(sourceWord, commenTargetHash);
		}
	}

	public void writeSourceCommen(ArrayList<Integer> commenSources, ArrayList<Integer> oftenTargets, ArrayList<Integer> commenTargets,
		HashMap<Integer, Integer> sourceTokenTimes, HashMap<Integer, Integer> targetTokenTimes) {

		//all commen source words
		for (Integer sourceWord : commenSources) {
			// all often target words.
			for (Integer targetWord : oftenTargets) {

				//the target word appears very often, it is nearly a stop words. such as ',',  'do'. so 
				//no commen source list for such source words
				int targetTimes;
				if (null != targetMultiNumHash.get(targetWord)) {
					targetTimes = targetMultiNumHash.get(targetWord);
				} else {
					targetTimes = 1;
				}
				if (targetTimes > 5 * sourceMultiNumHash.get(Math.min(sourceMultiNumHash.size() - 1, myConst.maxSM - 1))
					|| targetTimes > myConst.maxOftenCommenDiff * sourceMultiNumHash.get(Math.min(sourceWord, sourceMultiNumHash.size() - 1))) {
					continue;
				}

				HashMap<Integer, Integer> oftenTargetHash = sourceToTargetList.get(sourceWord);

				if (null == oftenTargetHash.get(targetWord)) {
					int times = Math.min(sourceTokenTimes.get(sourceWord), targetTokenTimes.get(targetWord));
					oftenTargetHash.put(targetWord, times);
				} else {
					int times = Math.min(sourceTokenTimes.get(sourceWord), targetTokenTimes.get(targetWord));
					int oldTimes = oftenTargetHash.get(targetWord);
					oftenTargetHash.put(targetWord, oldTimes + times);
				}
				sourceToTargetList.set(sourceWord, oftenTargetHash);
			}

			//all commen target words
			HashMap<Integer, Integer> commenTargetHash = sourceToTargetList.get(sourceWord);

			for (Integer targetWord : commenTargets) {

				if (null == commenTargetHash.get(targetWord)) {
					int times = Math.min(sourceTokenTimes.get(sourceWord), targetTokenTimes.get(targetWord));
					commenTargetHash.put(targetWord, times);
				} else {
					int times = Math.min(sourceTokenTimes.get(sourceWord), targetTokenTimes.get(targetWord));
					int oldTimes = commenTargetHash.get(targetWord);
					commenTargetHash.put(targetWord, oldTimes + times);
				}
			}
			sourceToTargetList.set(sourceWord, commenTargetHash);
		}
	}
}
