/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package diceMatrix.train;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import diceMatrix.doubleToString;
import diceMatrix.getSent;
import diceMatrix.myConst;
import diceMatrix.sentencePairDice;

/**
 * given a file pair, source and target files(with the alignment fild), get the
 * dice file for this file pair.
 *
 * @author Huiqin Körkel-Qu
 */
public class fileDiceTrain {

	sentencePairDice SentPairDice;

	public fileDiceTrain(sentencePairDice SentPairDice) {
		this.SentPairDice = SentPairDice;
	}

	public void get(File sourceFile, File targetFile, File alignFile, File diceFile) throws NumberFormatException, IOException {
		ArrayList<int[]> sourceAllSent = new ArrayList<int[]>(20000);
		ArrayList<int[]> targetAllSent = new ArrayList<int[]>(20000);

		// read the source file, target, and alignment file
		BufferedReader sourceReader = new BufferedReader(new FileReader(sourceFile));
		BufferedReader targetReader = new BufferedReader(new FileReader(targetFile));

		//write to dice file.
		BufferedWriter diceWriter = new BufferedWriter(new FileWriter(diceFile));;

		//get all source sentences

		String currentLine;
		while (null != (currentLine = sourceReader.readLine())) {
			//this is a sentence only contains stop words
			if (currentLine.length() == 0) {
				sourceAllSent.add(null);
				continue;
			}
			String[] currentStrings = currentLine.split("\\s+");
			int[] currentNums = new int[currentStrings.length];
			for (int i = 0; i < currentStrings.length; i++) {
				currentNums[i] = Integer.parseInt(currentStrings[i]);
			}
			sourceAllSent.add(currentNums);
		}
		sourceReader.close();
		sourceAllSent.trimToSize();
		//get all target sentences.
		while (null != (currentLine = targetReader.readLine())) {
			//this is a sentence only contains stop word.
			if (currentLine.length() == 0) {
				targetAllSent.add(null);
				continue;
			}

			String[] currentStrings = currentLine.split("\\s+");
//			System.out.println(currentLine);
			int[] currentNums = new int[currentStrings.length];
			for (int i = 0; i < currentStrings.length; i++) {
				currentNums[i] = Integer.parseInt(currentStrings[i]);
			}
			targetAllSent.add(currentNums);

		}
		targetReader.close();
		targetAllSent.trimToSize();
		//get alignment. only keep the 1;1 alignment.
		getAlignment getAligner = new getAlignment();
		getAligner.createAlignMap(alignFile);
		getAligner.getSingleAlign();
		//select from all the alignment 1;1 alignments.
		//T2,T3,S2,S3 are also in single alignment except T 
		ArrayList<Integer> sourceSingleArray = getAligner.sourceSingleArray;
		ArrayList<Integer> targetSingleArray = getAligner.targetSingleArray;
		ArrayList<String> singleAlignLabel = getAligner.singleAlignLabel;

		//if there is an source and  target alignment not be used, 
		//find its corresponding sequence.		
		while (sourceSingleArray.size() > 0 && targetSingleArray.size() > 0) {

			//if there is less than 15 F, there is a T appears, the sequence continues
			//otherwise break the sequence, start a new sequence from the single align array.

			//FNum counts how many continues F until now.
			int FNum = 0;

			//the start of a sequence is from the first alignment of the single alignments.
			int sourceStart = sourceSingleArray.get(0);
			int targetStart = targetSingleArray.get(0);
			String startLabel = singleAlignLabel.get(0);
			// if both are not the beginning of a text, add a sentence pair or not before the first sentence pair
			// according to a random value.
			//from the alignment array. 

			if (null != getSent.getSingleSent(sourceAllSent, sourceStart - 1)
				&& null != getSent.getSingleSent(targetAllSent, targetStart - 1)) {
				int[] sourceSent = getSent.getSingleSent(sourceAllSent, sourceStart - 1);
				int[] targetSent = getSent.getSingleSent(targetAllSent, targetStart - 1);
				double diceValue = this.SentPairDice.get(sourceSent, targetSent);
				double random = Math.random();
				//this avoid too many T2,T3,S2,S3 as start of  sequences. Fvor23
				if (random > 0.1 && (startLabel.contains("2") || startLabel.contains("3"))) {
					diceWriter.write(doubleToString.toBinaryFeature(diceValue, 2) + " "
						+ (sourceStart) + ";" + (targetStart) + " F" + "\n");
				}
				//to avoid all line start with T or T2,T3,S2,S3
//			    	if(random>0.1)
//				    	{
//				    	diceWriter.write(doubleToString.toBinaryFeature(diceValue, 2)+" "
//				    			+(sourceStart)+";"+(targetStart)+" F"+"\n");
//				    	}
			}

			//the max sequence  could contain all the sentences following..
			for (int i = 0; i < sourceAllSent.size() - sourceStart; i++) {
				//the current sent pair in sequence
				int sourceSentOrder = sourceStart + i;
				int targetSentOrder = targetStart + i;
				if (sourceSentOrder >= sourceAllSent.size() || targetSentOrder >= targetAllSent.size()) {
					break;
				}
				int[] sourceSent = getSent.getSingleSent(sourceAllSent, sourceSentOrder);
				int[] targetSent = getSent.getSingleSent(targetAllSent, targetSentOrder);

				//get the dice value for the current sent pair
				double diceValue = this.SentPairDice.get(sourceSent, targetSent);

				String label = "F";
				boolean findT = false;

				//to see if the current sent pair appears in the alignment.
				//if so, remove this alignment, and set the label to be T, and set the FNum to be 0.
				// if not. set the label to be 'F', and FNum increases.

				for (int j = 0; j < sourceSingleArray.size(); j++) {

					if (sourceSingleArray.get(j) == sourceSentOrder && targetSingleArray.get(j) == targetSentOrder) {
						findT = true;
						label = singleAlignLabel.get(j);
						sourceSingleArray.remove(j);
						targetSingleArray.remove(j);
						singleAlignLabel.remove(j);
						FNum = 0;
						break;
					}
				}
				if (!findT) {
					FNum += 1;

				}
				//the start of a sequence must be in the alignment, so remove it.
//			    	sourceSingleArray.remove(0);
//			    	targetSingleArray.remove(0);

				diceWriter.write(doubleToString.toBinaryFeature(diceValue, 2) + " "
					+ (sourceSentOrder + 1) + ";" + (targetSentOrder + 1) + " " + label + "\n");

				if (FNum == myConst.maxFNumInSeq) {
					diceWriter.write("\n");
					break;
				}
			}
		}
		diceWriter.close();
	}
}
