/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package diceMatrix.train;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

/**
 * get the alignment from alignment file. createAlignMap create two
 * arrayList<int[]> which save the non 0:n or n:0 alignments
 * getSingleAlign create two arrayList<Integer> which save the 1:1 alignment
 * only
 * if in the align file, the alignment is p;q, (sentence start form 1) the align
 * map save p-1;q-1 (sentence start from 0)
 *
 * @author Huiqin Körkel-Qu
 */
public class getAlignment {

	ArrayList<int[]> sourceList = new ArrayList<int[]>(10000);
	ArrayList<int[]> targetList = new ArrayList<int[]>(10000);
	ArrayList<Integer> sourceSingleArray = new ArrayList<Integer>(10000);
	ArrayList<Integer> targetSingleArray = new ArrayList<Integer>(10000);
	ArrayList<String> singleAlignLabel = new ArrayList<String>(10000);

	public boolean createAlignMap(File alignFile) throws IOException {

		if (alignFile.isDirectory() || !alignFile.exists()) {
			System.out.println("the alignment file " + alignFile.getAbsolutePath()
				+ " doesn't exist!");
			return false;
		}

		FileReader fr = new FileReader(alignFile);
		BufferedReader br = new BufferedReader(fr);
		String currentLine;

		while (null != (currentLine = br.readLine())) {
//				System.out.println("currentAlign line is "+currentLine);
			if (currentLine.length() <= 1 || currentLine.equals("\n")) {
				continue;
			}
			String[] alignSplit = new String[2];
			//1:0 or 0:1 alignment, omit
			if (currentLine.split(";").length < 2) {
				continue;
			}
			if (currentLine.split(";")[0].equals("")
				|| currentLine.split(";")[1].equals("")) {
				continue;
			}

			alignSplit[0] = currentLine.split(";")[0].trim();

			String[] source = alignSplit[0].split("\\s+");
			int[] sourceIds = new int[source.length];
			for (int i = 0; i < source.length; i++) {
				sourceIds[i] = Integer.parseInt(source[i]) - 1;
//					System.out.println("add source id "+sourceIds[i]+" to source side");
			}
			sourceList.add(sourceIds);

			//if one line only with source tag, without target tag, 
			//give the target tag the value ""
			try {
				alignSplit[1] = currentLine.split(";")[1].trim();


				String[] target = alignSplit[1].split("\\s+");
				int[] targetIds = new int[target.length];
				for (int i = 0; i < target.length; i++) {
					targetIds[i] = Integer.parseInt(target[i]) - 1;
//					System.out.println("add target id "+targetIds[i]+" to source side");
				}
				targetList.add(targetIds);
			} catch (Exception o) {
				int[] targetIds = new int[1];
				targetIds[0] = -1;
				targetList.add(targetIds);
//					System.out.println("target id is null");
			}
		}
		br.close();
		fr.close();

		targetList.trimToSize();
		sourceList.trimToSize();
//			System.out.println("alignment is ");
//			for(int i=0;i<sourceList.size();i++)
//			{
//				for(int j=0;j<sourceList.get(i).length;j++)
//				{
//					System.out.print(sourceList.get(i)[j]+" ");
//				}
//				System.out.print(";");
//				for(int j=0;j<targetList.get(i).length;j++)
//				{
//					System.out.print(targetList.get(i)[j]+" ");
//				}
//				System.out.println();
//			}
			
		return true;
	}

	public void getSingleAlign() {

		for (int i = 0; i < this.sourceList.size(); i++) {

			if (sourceList.get(i).length == 1 && targetList.get(i).length == 1) {
				sourceSingleArray.add(sourceList.get(i)[0]);
				targetSingleArray.add(targetList.get(i)[0]);
				singleAlignLabel.add("T");
			} else if (sourceList.get(i).length == 2 && targetList.get(i).length == 1) {
				sourceSingleArray.add(sourceList.get(i)[1]);
				targetSingleArray.add(targetList.get(i)[0]);
				singleAlignLabel.add("S2");
			} else if (sourceList.get(i).length == 3 && targetList.get(i).length == 1) {
				sourceSingleArray.add(sourceList.get(i)[2]);
				targetSingleArray.add(targetList.get(i)[0]);
				singleAlignLabel.add("S3");
			} else if (sourceList.get(i).length == 1 && targetList.get(i).length == 2) {
				sourceSingleArray.add(sourceList.get(i)[0]);
				targetSingleArray.add(targetList.get(i)[1]);
				singleAlignLabel.add("T2");
			} else if (sourceList.get(i).length == 1 && targetList.get(i).length == 3) {
				sourceSingleArray.add(sourceList.get(i)[0]);
				targetSingleArray.add(targetList.get(i)[2]);
				singleAlignLabel.add("T3");
			} else {
			}
		}
	}

	public static void main(String[] argcs) throws IOException {
		getAlignment get = new getAlignment();
		File alignFile = new File("/home/students/huiqin/gutenberg/forDice/align/nabot.align");
		get.createAlignMap(alignFile);
	}
}
