/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package diceMatrix.train;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import diceMatrix.myConst;
import diceMatrix.wordToNum;

/**
 * from file pairs with alignment got the dice matrix and dice array.
 *
 * @author Huiqin Körkel-Qu
 *
 */
public class rawFileToDiceMatrix {

	public static void main(String[] args) throws IOException {

		String filePath = args[0];
		String sourcePath = filePath + "de/";
		String targetPath = filePath + "en/";
		String alignPath = filePath + "align/";
		String sourceNumPath = filePath + "num/de/";
		String targetNumPath = filePath + "num/en/";
		String stopWordPath = "";
		String stopSourceWordFile = stopWordPath + "stopwords.de";
		String stopTargetWordFile = stopWordPath + "stopwords.en";

		writeDiceFiles.createDir(sourceNumPath);
		writeDiceFiles.createDir(targetNumPath);

		wordCount sourceCounter = new wordCount();
		wordCount targetCounter = new wordCount();

		String outputSourceCount = filePath + "source.count";
		String outputTargetCount = filePath + "target.count";
		//count the tokens and how many  times they appears

//		 long wordCounterStart=System.currentTimeMillis();

		File sourceDir = new File(sourcePath);
		if (null == sourceDir) {
			System.out.println("the source Path doesn't exists!");
			System.exit(1);
		}
		File targetDir = new File(targetPath);
		if (null == targetDir) {
			System.out.println("the target Path doesn't exists!");
			System.exit(1);
		}

		File[] alignFileList = new File(alignPath).listFiles();
//		    System.out.println(alignPath);
//		    System.out.println();
		if (null == alignFileList || alignFileList.length == 0) {

			System.out.println("the alignmentPath is empty!");
			System.out.println(alignPath);
			System.exit(1);
		}
//		    for(File file:alignFileList)
//		    {
//		    	System.out.println(file.getAbsolutePath());
//		    }

		int countFileNum = 0;

		File[] sourceFiles = sourceDir.listFiles();
//			System.out.println("there are "+sourceFiles.length +"files in source dir "+sourceDir.getAbsolutePath());
		for (int i = 0; i < sourceFiles.length; i++) {
			String fileName = sourceFiles[i].getName();
			if (!("." + fileName.split("\\.", 2)[1]).equals(diceMatrix.myConst.extensionForDice)) {
				continue;
			}
//			    System.out.println("file now is "+fileName);;
			File targetFile = new File(targetPath + fileName);

			File alignFile = new File(alignPath + fileName.split("\\.")[0] + diceMatrix.myConst.extensionForAlign);
			if (!alignFile.isFile() || !targetFile.isFile()) {
				continue;
			}

			sourceCounter.count(sourceFiles[i], stopSourceWordFile, outputSourceCount);
			targetCounter.count(targetFile, stopTargetWordFile, outputTargetCount);
			countFileNum += 1;
		}

		if (countFileNum == 0) {
			System.out.println("filePath: " + filePath); //E
			System.out.println("Error! no file pair is counted to get dice. ");
			System.exit(1);
		}
		//separte the tokens that appear only once from tokens that appear more than once.
		sourceCounter.cutSingleWords();
		targetCounter.cutSingleWords();
		//get the hashMap of tokens that appear more than once.
		//String is the token, int[0] is the corresponing int of token, int[1] saves how many time this token appears in training. 
		HashMap<String, int[]> sourceMultiWordsTimesHash = sourceCounter.multiWordsTimesHash;
		HashMap<String, int[]> targetMultiWordsTimesHash = targetCounter.multiWordsTimesHash;
		//change each token(which appear mor than once) int to num, and save how many times it appears.
		HashMap<Integer, Integer> sourceMultiNumHash = sourceCounter.getNumTimesHash(sourceMultiWordsTimesHash);
		HashMap<Integer, Integer> targetMultiNumHash = sourceCounter.getNumTimesHash(targetMultiWordsTimesHash);
		//get the list of the tokens that appear only once. it is sorted alphabet..
		String[] sourceSingle = sourceCounter.singleWords;
		String[] targetSingle = targetCounter.singleWords;
		// stop word list.
		HashMap<String, Integer> stopWordsSource = sourceCounter.stopWordsList;
		HashMap<String, Integer> stopWordsTarget = targetCounter.stopWordsList;

//			    System.out.println("sourcewordsListNameHash size is:"+sourceMultiWordsTimesHash.size());
//			    System.out.println("targetwordsListNameHash size is:"+targetMultiWordsTimesHash.size());
//			    System.out.println("source single word num is "+sourceCounter.singleWords.length);
//			    System.out.println("target single word num is "+targetCounter.singleWords.length);

		// if the tokens are less than some level, change this level to be the number of tokens
		if (sourceMultiWordsTimesHash.size() < myConst.maxSM) {
			myConst.maxSM = sourceMultiWordsTimesHash.size();
			myConst.fastSourceWordsNum = sourceMultiWordsTimesHash.size();
		} else if (sourceMultiWordsTimesHash.size() < myConst.fastSourceWordsNum) {

			myConst.fastSourceWordsNum = sourceMultiWordsTimesHash.size();
		} else {
		}

		if (targetMultiWordsTimesHash.size() < myConst.maxTM) {
			myConst.maxTM = targetMultiWordsTimesHash.size();
			myConst.fastTargetWordsNum = targetMultiWordsTimesHash.size();
		} else if (targetMultiWordsTimesHash.size() < myConst.fastTargetWordsNum) {

			myConst.fastTargetWordsNum = targetMultiWordsTimesHash.size();
		} else {
		}

//			    long wordCounterEnd=System.currentTimeMillis();
//			    System.out.println("count the words costed "+(wordCounterEnd-wordCounterStart)/1000 +" seconds");

		diceMatrixList diceCounter = new diceMatrixList(sourceMultiNumHash, targetMultiNumHash, sourceSingle, targetSingle);

//	    long diceStart=System.currentTimeMillis();

		wordToNum sourceChanger = new wordToNum(sourceMultiWordsTimesHash, sourceSingle, stopWordsSource);
		wordToNum targetChanger = new wordToNum(targetMultiWordsTimesHash, targetSingle, stopWordsTarget);

		for (int i = 0; i < sourceFiles.length; i++) //	    for(int i=0;i<5;i++)
		{
			String fileName = sourceFiles[i].getName();
//		    System.out.println(sourceFiles[i].getAbsolutePath());

			if (!("." + fileName.split("\\.", 2)[1]).equals(diceMatrix.myConst.extensionForDice)) {
//		    	System.out.println("the file "+fileName +"   is not for dice file!");
				continue;
			}
//		    System.out.println("source file is "+sourceFiles[i].getAbsolutePath());
			File targetFile = new File(targetPath + fileName);
			File alignFile = new File(alignPath + fileName.split("\\.", 2)[0] + diceMatrix.myConst.extensionForAlign);
//		    System.out.println("alignFile is:"+alignFile.getAbsolutePath());
			if (!alignFile.isFile() || !targetFile.isFile()) {
//		    	System.out.println("align file or target file does not exist");
				continue;
			}

//		    long start=System.currentTimeMillis();
			//change source and target file in to ArrayList<int[]>
			// the ith element of ArrayList is the ith sentence
			String sourceNumFileName = sourceNumPath + sourceFiles[i].getName().split("\\.")[0].concat(".num");
			String targetNumFileName = targetNumPath + targetFile.getName().split("\\.")[0].concat(".num");
			File sourceNumFile = new File(sourceNumFileName);
			File targetNumFile = new File(targetNumFileName);
			ArrayList<int[]> sourceArray = sourceChanger.changeFileToArray(sourceFiles[i], sourceNumFile);
			ArrayList<int[]> targetArray = targetChanger.changeFileToArray(targetFile, targetNumFile);

			//get the alignment from alignment file.  
			getAlignment alignment = new getAlignment();
			alignment.createAlignMap(alignFile);
			ArrayList<int[]> sourceAlignment = alignment.sourceList;
			ArrayList<int[]> targetAlignment = alignment.targetList;

			diceCounter.calculateAppearance(sourceArray, targetArray, sourceAlignment, targetAlignment);

			sourceAlignment.clear();
			targetAlignment.clear();

//			long end=System.currentTimeMillis();
//			System.out.println((i+1)+" files haven been finished");
//	         System.out.println("for file "+fileName+ "\nthe dice counter costed  "+(end-start)+"mini seconds");
		}

		File matrixFile = new File(filePath + "dice.matrix");
		int[][] diceMatrix = diceCounter.diceMatrix;
		writeDiceFiles.writeMatrix(diceMatrix, matrixFile);

		ArrayList<HashMap<Integer, Integer>> sourceToTarget = diceCounter.sourceToTargetList;

		File arrayFile = new File(filePath + "source.array");

//		long arrayStart=System.currentTimeMillis();

		writeDiceFiles.writeArray(sourceToTarget, arrayFile);
//		long arrayEnd=System.currentTimeMillis();
//		System.out.println("save array file costed  "+(arrayEnd-arrayStart)/1000 +" seconds");

//		long diceEnd=System.currentTimeMillis();
//        System.out.println("all files  dice counter costed  "+(diceEnd-diceStart)/1000 +" seconds");

	}
}
