/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package generalTest;

import java.io.File;
import java.io.IOException;

/**
 * crfTest, which use beam search.
 *
 * @author Huiqin Körkel-Qu
 */
public class crfTest {

	String testPath;
	static String modelName;
	static String trainingResultPath;
	static String outputAlignmentExtension; //E

	/**
	 * argc contains at least 2 parameters.
	 *
	 * argc[0] is the path of test files(testPath)file, the source test file is
	 * saved under testPath/de/ the target test file is saved under testPath/en/
	 * the crf test runs only iff both source and target are there.
	 *
	 *
	 * argc[1] is the path where the result of training is saved. it should
	 * contain dice.matrix, source.array, source.countmulti,
	 * source.countsingle,target.countmulit,target.countsingle which will be
	 * used to get the dice values for the test files
	 *
	 * argc[2] is the name of the model got from crf training. this model shows
	 * the weight of each feature, and will be used in crf(wapiti ) label for
	 * the test file pair.
	 *
	 * if argc[2] is not given, it should be under the argc[1] with name model
	 *
	 * the result of crf label is saved under the sourcePath, which has the same
	 * name as source and target files, and with appendix name .alignment
	 */
	public crfTest(String argc[]) {
		//E
		if (argc.length < 3) {
			System.out.println("parameters is not enought for label!");
			System.exit(1);
		} else if (argc.length == 3) {
			testPath = argc[0];
			if (testPath.charAt(testPath.length() - 1) != '/') {
				testPath = testPath + "/";
			}

			trainingResultPath = argc[1];
			if (trainingResultPath.charAt(trainingResultPath.length() - 1) != '/') {
				trainingResultPath = trainingResultPath + "/";
			}

			outputAlignmentExtension = argc[2];

			modelName = trainingResultPath + "model";

		} else {
			testPath = argc[0];
			if (testPath.charAt(testPath.length() - 1) != '/') {
				testPath = testPath + "/";
			}

			trainingResultPath = argc[1];
			if (trainingResultPath.charAt(trainingResultPath.length() - 1) != '/') {
				trainingResultPath = trainingResultPath + "/";
			}
			outputAlignmentExtension = argc[2];
			modelName = argc[3];

		}

//		 if(argc.length<2)
//		 {
//			 System.out.println("parameters is not enought for label!");
//			 System.exit(1);
//		 }
//		 else if(argc.length==2)
//		 {
//			 testPath=argc[0];
//			 if(testPath.charAt(testPath.length()-1)!='/')
//				{
//					testPath=testPath+"/";
//				}
//			 
//			 trainingResultPath=argc[1];
//			 if(trainingResultPath.charAt(trainingResultPath.length()-1)!='/')
//				{
//					trainingResultPath=trainingResultPath+"/";
//				}
//			 modelName=trainingResultPath+"model";
//			 
//		 }
//		 
//		 else
//		 {
//			 testPath=argc[0];
//			 if(testPath.charAt(testPath.length()-1)!='/')
//				{
//					testPath=testPath+"/";
//				}
//			 
//			 trainingResultPath=argc[1];
//			 if(trainingResultPath.charAt(trainingResultPath.length()-1)!='/')
//				{
//					trainingResultPath=trainingResultPath+"/";
//				}
//			 modelName=argc[2];
//			 
//		 }

	}

	public static void main(String[] argcs) throws IOException, NumberFormatException, InterruptedException {
		crfTest tester = new crfTest(argcs);
		String sourcePath = tester.testPath + "de/";
		String targetPath = tester.testPath + "en/";

		File sourceDir = new File(sourcePath);

		if (!sourceDir.isDirectory()) {
			System.out.println("source test path error!");
			System.exit(1);
		}

		File targetDir = new File(targetPath);

		if (!targetDir.isDirectory()) {
			System.out.println("target test path error!");
			System.exit(1);
		}

		File sourceFiles[] = sourceDir.listFiles();
		if (!new File(modelName).isFile()) {
			System.out.println("the model is not there!");
			System.exit(1);
		}

		for (File sourceFile : sourceFiles) {

			File targetFile = new File(targetPath + sourceFile.getName());
			if (!targetFile.isFile()) {
				continue;
			} else {
				if (sourceFile.getName().contains(diceMatrix.myConst.extensionForFeature)) {
					beamSearch beamer = new beamSearch(trainingResultPath, modelName,
						sourceFile.getAbsolutePath(), targetFile.getAbsolutePath(),
						sourceFile.getAbsolutePath().split("\\.")[0] + ".feature");
					beamer.extendBeam(0, Math.min(beamer.maxSId / 20, 50), 0, Math.min(beamer.maxTId / 20, 50), modelName);
					beamer.writeAlignments(tester.outputAlignmentExtension); //E

				}
			}
		}
	}
}
