/**
 * CRFalign: A CRF-based sentence aligner for parallel corpora.
 *
 * Copyright (C) 2012 Eva Mujdricza-Maydt Copyright (C) 2012 Huiqin Körkel-Qu
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Developers: - Eva Mujdricza-Maydt (mujdricz@cl.uni-heidelberg.de) - Huiqin
 * Körkel-Qu (huiqin@cl.uni-heidelberg.de)
 */
package generalTrain;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import general.com.posMap.*;

/**
 * create feature file for every pair of training files. and merge all the
 * feature files to be training.txt, which will be used in wapiti crf training
 * process to get the model
 *
 * @author Huiqin Körkel-Qu
 */
public class generalTraining {

	String POSFileName;
	String dicePath;
	String sourcePath;
	String targetPath;
	String featurePath;
	String path;

	public generalTraining(String path) {

		if (path.charAt(path.length() - 1) != '/') {
			path = path + "/";
		}
		this.path = path;
		sourcePath = this.path + "de/";
		targetPath = this.path + "en/";
		featurePath = this.path + "feature/";
		dicePath = this.path + "dice/";
		POSFileName = "posMapDeEnPure";

	}

	/**
	 * get the feature files
	 *
	 * @return
	 * @throws IOException
	 */
	public int train() throws IOException {
		// get the POS map for POS feature.
		posHashtable posHash = new posHashtable();
		Hashtable<String, ArrayList<String>> posMap = posHash.createPosMapTable("de", "en", POSFileName);
		if (null == posMap) {
			System.out.println("the POS Map is empty");
		}
		File featureDir = new File(this.featurePath);
		if (!featureDir.isDirectory()) {

			boolean sucess = featureDir.mkdirs();
			if (!sucess) {
//			 System.out.println("unsuccessful in creating feature directory!");
				return 1;
			}
		}

		try {

			File dirDe = new File(sourcePath);
//	   System.out.println("source file path is"+sourcePath);
			File[] filesDe = dirDe.listFiles();
			if (null == filesDe) {
//    	   System.out.println("source file dir is not found or empty.");
//    	   System.out.println(sourcePath);

				return 1;
			}

			long start = System.currentTimeMillis();
			int featureFileNum = 0;
			for (int i = 0; i < filesDe.length; i++) //       for(int i=0;i<20;i++)
			{
				if (!("." + filesDe[i].getName().split("\\.", 2)[1]).equals(diceMatrix.myConst.extensionForFeature)) {
					continue;
				}
				//check if the corresponding english file exists. 
				File enFile = new File(targetPath + filesDe[i].getName());
				if (!enFile.isFile()) {
					continue;
				}
				String FileName = filesDe[i].getName().split("\\.")[0];

				/*
				 * get feature file for one training file pair.
				 */
				filePair stPair = new filePair(FileName, path,
					posMap);
				boolean featureFileSucceed = stPair.createFeature();
				/*
				 * it happens when the corresponding dice file does not exist,
				 * or not all sentences exist(improper xml file, etc. )
				 */
				if (featureFileSucceed == false) {
					continue;
//         	System.out.println("unsuccessful to create features for file pair"+filesDe[i].getName()+"!");
				}
				featureFileNum += 1;
				if (0 == featureFileNum % 5) {
					System.out.println(featureFileNum + " feature files are created");
				}
			}
			long end = System.currentTimeMillis();
			System.out.println("it costs " + (end - start) / 1000 + " seconds to create the training feature file.");

		} catch (Exception e) {
			e.printStackTrace();
			return 1;
		}
		return 0;
	}

	public static void main(String[] args) throws IOException, InterruptedException {

		generalTraining gutenbergReader = new generalTraining(args[0]);

		int run = gutenbergReader.train();
		if (run != 0) {
			System.out.println("Error in training, Training process stops!");
			System.exit(1);
		} else {

			// remove the dice directory which will not be used any more
//    	String moveDiceCommand="rm -r "+args[0]+"dice/";
//    	Process moveDiceDir=Runtime.getRuntime().exec(moveDiceCommand);
//    	moveDiceDir.waitFor();
//    	
			// merge all the feature file to be training.txt
			//which will be used as training files in wapiti train.
			String mergeCommand = "cat " + args[0] + "feature/* > " + args[0] + "training.txt";
//     	System.out.println("merge command is: "+mergeCommand);
			LinuxInteractor.executeCommand(mergeCommand, false);

			System.out.println("get full traing feature file");

			//remove the feature directory which will not be used any more.
//    	String moveFeatureCommand="rm -r "+args[0]+"feature/";
//    	Process moveFeatureDir=Runtime.getRuntime().exec(moveFeatureCommand);
//    	moveFeatureDir.waitFor();

		}
	}
}
